#ifndef __KYSEC_MODULE_AUTHORIZE_H__
#define __KYSEC_MODULE_AUTHORIZE_H__

#include <bits/types.h>
#include <linux/limits.h>


#define KMA_ERROR_OPERATOR_FAILED          (-1)
#define KMA_ERROR_INVALID_PARAMETER        (-2)
#define KMA_ERROR_GET_PROGRAM_PATH         (-3)
#define KMA_ERROR_GET_CONF_FAILED          (-4)
#define KMA_ERROR_SET_CONF_FAILED          (-5)
#define KMA_ERROR_ADD_CONF_KEY_FAILED      (-6)
#define KMA_ERROR_DEL_CONF_KEY_FAILED      (-7)
#define KMA_ERROR_FREE_FAILED              (-8)
#define KMA_ERROR_OUT_OF_MEMORY            (-9)
#define KMA_ERROR_DIRTY_MEMORY             (-10)

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
    KMA_PERMISSION_DEVCTL=0,
    KMA_PERMISSION_FPRO,
    KMA_PERMISSION_PPRO,
    KMA_PERMISSION_PBLK,
    KMA_PERMISSION_KMODPRO,
    KMA_PERMISSION_FIREWALL,
    KMA_PERMISSION_UNINSTALLPRO,
    KMA_PERMISSION_BGSET,
    KMA_PERMISSION_NETFLOW,
    KMA_PERMISSION_MAX_NUM,
} kma_permission_type_e;

static const char * const module_array[KMA_PERMISSION_MAX_NUM] = {
    "devctl",
    "fpro",
    "ppro",
    "pblk",
    "kmodpro",
    "firewall",
    "uninstallpro",
    "bgset",
    "netflow",
};

typedef struct {
    char program_path[PATH_MAX];
    void * pnext;
}kma_program_list_t;


typedef struct {
    char module_name[PATH_MAX];
    void * pnext;
}kma_module_info_t;

typedef struct {
    char program_path[PATH_MAX];
    kma_module_info_t * module_list;
    void * pnext;
}kma_program_info_t;

typedef struct {
    char package[PATH_MAX];
    kma_program_info_t * program_list;
    void * pnext;
}kma_package_info_t;


/*
检查涉密授权状态
* 输入参数：
*  无
* 输出参数：
*  无

* 返回值：
*  0：  当前非涉密授权状态
*  1：  当前是涉密授权状态
*/
int kma_check_sm_enable();


/*
检查当前是否已经授权某种安全权限
* 输入参数：
*  type： 安全权限类型
* 输出参数：
*  无

* 返回值：
*  0：  当前没有授权此安全权限
*  1：  当前已经授权此安全权限
*/
int kma_check_sm_has_permission(kma_permission_type_e type);


/*
查询指定的pid的进程是否具备某类安全接口操作权限
* 输入参数：
*  pid：指定程序的pid
*  type：指定的安全接口类型,参考kma_permission_type_e
* 输出参数：
*  permission：指定程序授权状态，0-指定程序没有被授权，1-指定程序已经被授权。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/

int kma_get_permission_bypid(pid_t pid,kma_permission_type_e type,int * permission);


/*
查询指定绝对路径的进程是否具备某类安全接口操作权限
* 输入参数：
*  path：指定程序的绝对路径
*  type：指定的安全接口类型，参考kma_permission_type_e
* 输出参数：
*  permission：指定程序授权状态，0-指定程序没有被授权，1-指定程序已经被授权。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/

int kma_get_permission_bypath(const char * path,kma_permission_type_e type,int * permission);

/*
增加指定package的指定进程的指定类型的安全接口授权
* 输入参数：
*  package：指定的软件包名
*  program_path：指定可执行文件路径
*  type：指定的安全接口类型，参考kma_permission_type_e
*  third_party:第三方安全套件标识， 0-指定软件包不是第三方安全套件，指定软件包是第三方套件
* 输出参数：
* 无。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kma_add_permission(const char * package,const char * program_path,int third_party,kma_permission_type_e type);


/*
增加指定package的指定进程的指定类型的安全接口授权拓展函数
* 输入参数：
*  package：指定的软件包名
*  program_path：指定可执行文件路径
*  module_type：指定的安全接口类型名称
*  third_party:第三方安全套件标识， 0-指定软件包不是第三方安全套件，指定软件包是第三方套件
*  enable:授权开关标志
* 输出参数：
* 无。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kma_add_permission_ext(const char * package,const char * program_path,int third_party,int enable,const char *module_name);

/*
删除指定package的指定进程的指定类型的安全接口授权
* 输入参数：
*  package：指定的软件包名
*  program_path：指定可执行文件路径
*  type：指定的安全接口类型，参考kma_permission_type_e
* 输出参数：
*  无。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kma_del_permission(const char * package,const char * program_path,kma_permission_type_e type);


/*
删除指定package的安全模块接口授权
* 输入参数：
*  package：指定的软件包名
* 输出参数：
*  无。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kma_del_package(const char * package);

/*
删除指定可执行文件的安全模块接口授权
* 输入参数：
*  package：指定的软件包名
* 输出参数：
*  无。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kma_del_program(const char * package,const char * program_path);

/*
查询第三方程序中指定类型安全接口授权程序列表
* 输入参数：
*  type：指定的安全接口类型,参考kma_permission_type_e
* 输出参数：
*  list：授权程序列表,如果授权列表返回的指针为空，则说明第三方程序无此类型的授权，否则说明第三方程序有授权。list指针的指向的内存由
*  函数内部申请，在调用方使用完list指针后，需要调用方主动调用kma_free_program_list函数释放内存。
* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kma_get_third_party_permission(kma_permission_type_e type,kma_program_list_t ** list);

/*
释放授权程序列表接口
* 输入参数：
*  list：授权程序列表指针
* 输出参数：
*  无
* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kma_free_program_list(kma_program_list_t *list);


/*
查询第三方软件包信息
* 输入参数：
*  无
* 输出参数：
*  package_info：。第三方安全套件授权信息，package_info指针的指向的内存由
*  函数内部申请，在调用方使用完list指针后，需要调用方主动调用kma_free_program_list函数释放内存。
* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kma_get_third_party_package_info(kma_package_info_t** package_info);

/*
释放package_info接口
* 输入参数：
*  package_info：package_info指针
* 输出参数：
*  无
* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kma_free_package_info(kma_package_info_t *package_info);

/*
按条件查询第三方软件包信息
* 输入参数：
*  type：授权类型、查询条件
*  third_party：第三方软件标志、查询条件
*  enable：  授权开关标志、查询条件
* 输出参数：
*  package_info：。第三方安全套件授权信息，package_info指针的指向的内存由
*  函数内部申请，在调用方使用完package_info指针后，需要调用方主动调用kma_free_package_info函数释放内存。
* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kma_get_package_info_by_condition(kma_permission_type_e type,int third_party,int enable,kma_package_info_t** package_info);


/*
按条件查询软件包授权信息
* 输入参数：
*  conf_path：授权文件路径
*  third_party：第三方软件标志、查询条件
*  enable：  授权开关标志、查询条件
* 输出参数：
*  package_info：。软件包授权信息，package_info指针的指向的内存由
*  函数内部申请，在调用方使用完package_info指针后，需要调用方主动调用kma_free_package_info函数释放内存。
* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kma_get_package_info_by_path(const char *conf_path,int third_party,int enable,kma_package_info_t** package_info);


/*
查询一个程序是否是第三方安全程序
* 输入参数：
*  program_path：指定的程序的绝对路径
* 输出参数：
*  is_third_party：是否是第三方安全程序，0-不是第三方安全程序，1-是第三方安全程序。
* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kma_is_third_party_program(const char * program_path,int *is_third_party);


#ifdef __cplusplus
}
#endif

#endif
